@@ -1,210 +1,43 @@
-from .paystack_config import *
-from .errors import *
-from .customers import Customer
-from .plans import Plan
-from .filters import Filter
-from datetime import datetime
-import requests
+'''
+Managers.py
+'''
+
 import json
-import jsonpickle
+import requests
 import validators
-import math
+from .base import Manager
+from .customers import Customer
+from .errors import APIConnectionFailedError, InvalidEmailError, URLValidationError
+from .filters import Filter
+from .paystack_config import PaystackConfig
+from .plans import Plan
+from .transfers import Transfer
+from .transactions import Transaction
 
-class Manager():
+class TransactionsManager(Manager):
     '''
-    Abstract base class for 'Manager' Classes
-    '''
-
-    PAYSTACK_URL = None
-    SECRET_KEY = None
-    LOCAL_COST = None
-    INTL_COST = None
-    PASS_ON_TRANSACTION_COST = None
-
-    decoder = json.JSONDecoder()
-
-    def __init__(self, *args, **kwargs):
-        if type(self) is Manager:
-            raise TypeError("Can not make instance of abstract base class")
-
-        if not PaystackConfig.SECRET_KEY or not PaystackConfig.PUBLIC_KEY:
-            raise ValueError("No secret key or public key found,"
-                             "assign values using PaystackConfig.SECRET_KEY = SECRET_KEY and"
-                             "PaystackConfig.PUBLIC_KEY = PUBLIC_KEY")
-
-        self.PAYSTACK_URL = PaystackConfig.PAYSTACK_URL
-        self.SECRET_KEY = PaystackConfig.SECRET_KEY
-
-        if type(self) is PaymentManager:
-            self.LOCAL_COST = PaystackConfig.LOCAL_COST
-            self.INTL_COST = PaystackConfig.INTL_COST
-            self.PASS_ON_TRANSACTION_COST = PaystackConfig.PASS_ON_TRANSACTION_COST
-
-
-    def get_content_status(self, content):
-        '''
-        Method to return the status and message from an API response
-
-        Arguments :
-        content : Response as a dict
-        '''
-
-        if  not type(content) is dict:
-            raise TypeError("Content argument should be a dict")
-
-        return (content['status'], content['message'])
-
-    def parse_response_content(self, content):
-        '''
-        Method to convert a response's content in bytes to a string.
-
-        Arguments:
-        content : Response in bytes
-        '''
-        content = bytes.decode(content)
-        content = self.decoder.decode(content)
-        return content
-
-    def build_request_args(self, data={}):
-        '''
-        Method for generating required headers.
-        Returns a tuple containing the generated headers and the data in json.
-
-        Arguments :
-        data(Dict) : An optional data argument which holds the body of the request.
-        '''
-        headers = {'Authorization' : 'Bearer %s' % self.SECRET_KEY,
-                   'Content-Type' : 'application/json',
-                   'cache-control' : 'no-cache'
-                  }
-
-        data = json.dumps(data)
-
-        return (headers, data)
-
-    def toJSON(self):
-        '''
-        Method for serializing an instance of a class
-        '''
-        return jsonpickle.encode(self)
-
-class PaymentManager(Manager):
-    '''
-    PaymentManager class that handles every part of a transaction
+    TransactionsManager class that handles every part of a transaction
 
     Attributes:
-    __amount : Transaction cost
-    __email : Buyer's email
-    __reference
-    __authorization_url
+    amount : Transaction cost
+    email : Buyer's email
+    reference
+    authorization_url
     card_locale : Card location for application of paystack charges
     '''
 
-    __amount = None
-    __email = None
-    __reference = None
-    __access_code = None
-    __authorization_url = None
+    LOCAL_COST = PaystackConfig.LOCAL_COST
+    INTL_COST = PaystackConfig.INTL_COST
+    PASS_ON_TRANSACTION_COST = PaystackConfig.PASS_ON_TRANSACTION_COST
+
     __endpoint = '/transaction'
-    card_locale = None
-
-    def __init__(self, amount: int, email, reference='', access_code='',
-                 authorization_url='', card_locale='LOCAL', endpoint='/transaction'):
+    def __init__(self, endpoint='/transaction'):
         super().__init__(self)
-        try:
-            amount = int(amount)
-        except ValueError as error:
-            raise ValueError("Invalid amount. Amount(in kobo) should be an integer")
-            #Error message
-        else:
-            #Check if the provided email is valid
-            if validators.email(email):
-                self.__amount = amount
-                self.__email = email
-                self. __reference = reference
-                self.__access_code = access_code
-                self.__authorization_url = authorization_url
-                self.card_locale = card_locale
-            else:
-                raise InvalidEmailError
+        self.__endpoint = endpoint
 
 
-
-    #Transaction amount property
-    @property
-    def amount(self):
-        return self.__amount
-
-    #Transaction email property
-    @property
-    def email(self):
-        return self.__email
-
-    @property
-    def authorization_url(self):
-        return self.__authorization_url
-
-    @property
-    def reference(self):
-        return self.__reference
-
-    @property
-    def access_code(self):
-        return self.access_code
-
-    def generate_reference_code(self):
-        '''
-        Generates a unique transaction reference code
-        '''
-        date = datetime.now()
-        year = date.year
-        month = str(date.month).zfill(2)
-        day = str(date.day).zfill(2)
-        date_stamp = "%s%s%s" % (year, month, day)
-
-        reference_code = "%s%s" % (date_stamp, hash(self.email))
-
-        time = date.time()
-        hour = time.hour
-        minute = time.minute
-        second = time.second
-
-        reference_code += "%s%s%s" % (hour, minute, second)
-
-        return reference_code
-
-    def full_transaction_cost(self, locale):
-        '''
-        Adds on paystack transaction charges and returns updated cost
-
-        Arguments:
-        locale : Card location (LOCAL or INTERNATIONAL)
-        '''
-        if self.amount:
-
-            if locale not in ('LOCAL', 'INTERNATIONAL'):
-                raise ValueError("Invalid locale, locale should be 'LOCAL' or 'INTERNATIONAL'")
-
-            else:
-                locale_cost = {'LOCAL' : self.LOCAL_COST, 'INTERNATIONAL' : self.INTL_COST}
-
-                cost = self.amount / (1 - locale_cost[locale])
-
-                if cost > 250000:
-                    cost = (self.amount + 100)/ (1 - locale_cost[locale])
-
-                paystack_charge = locale_cost[locale] * cost
-                #Paystack_charge is capped at N2000
-                if paystack_charge > 200000:
-                    cost = self.amount + 200000
-
-                return math.ceil(cost)
-
-        else:
-            raise AttributeError("Amount not set")
-
-    def start_transaction(self, method, callback_url='', metadata={},
-                          plan_code=None, endpoint='/initialize'):
+    def initialize_transaction(self, method, transaction: Transaction,
+                               callback_url='', endpoint='/initialize'):
         '''
         Initializes a paystack transaction.
         Returns an authorization url which points to a paystack form if the method is standard.
@@ -221,35 +54,23 @@ class PaymentManager(Manager):
         if method not in ('STANDARD', 'INLINE', 'INLINE EMBED'):
             raise ValueError("method argument should be STANDARD, INLINE or INLINE EMBED")
 
-
         if self.PASS_ON_TRANSACTION_COST:
-            self.__amount = self.full_transaction_cost(self.card_locale)
+            transaction.amount = transaction.full_transaction_cost(transaction.card_locale)
 
-        amount = self.__amount
-        email = self.__email
+        data = json.JSONDecoder().decode(transaction.to_json())
 
-        data = {'amount' : amount, 'email' : email, 'reference' : self.generate_reference_code(),
-                'metadata' : metadata}
+        if callback_url:
+            if validators.url(callback_url):
+                data['callback_url'] = callback_url
 
-        if plan_code:
-            data['plan'] = plan_code
+            else:
+                raise URLValidationError
 
         if method in ('INLINE', 'INLINE EMBED'):
             data['key'] = PaystackConfig.PUBLIC_KEY
             return data
 
-        if callback_url:
-            #Check if callback_url is a valid url
-            if validators.url(callback_url):
-                data['callback_url'] = callback_url
-                headers, data = self.build_request_args(data)
-
-            else:
-                raise URLValidationError
-
-        else:
-            #Use callback provided on paystack dashboard
-            headers, data = self.build_request_args(data)
+        headers, data = self.build_request_args(data)
 
         url = self.PAYSTACK_URL + self.__endpoint + endpoint
         response = requests.post(url, headers=headers, data=data)
@@ -262,18 +83,13 @@ class PaymentManager(Manager):
         #status = True for a successful connection
         if status:
             data = content['data']
-            self.__reference = data['reference']
-            self.__access_code = data['access_code']
-            self.__authorization_url = data['authorization_url']
-            return self.__authorization_url
-
+            transaction = Transaction.from_json(data)
+            return transaction
         else:
             #Connection failed
             raise APIConnectionFailedError(message)
 
-
-
-    def verify_transaction(self, reference, endpoint='/verify/'):
+    def verify_transaction(self, transaction: Transaction, endpoint='/verify/'):
         '''
         Verifies a payment using the transaction reference.
 
@@ -281,7 +97,7 @@ class PaymentManager(Manager):
         endpoint : Paystack API endpoint for verifying transactions
         '''
 
-        endpoint += reference
+        endpoint += transaction.reference
         url = self.PAYSTACK_URL + self.__endpoint + endpoint
 
         headers, _ = self.build_request_args()
@@ -291,33 +107,101 @@ class PaymentManager(Manager):
 
         status, message = self.get_content_status(content)
 
+        if status:
+            data = content['data']
+            transaction = Transaction.from_json(data)
+            transaction.email = data['customer']['email']
+            transaction.authorization_code = data['authorization']['authorization_code']
+            return transaction
+        else:
+            raise APIConnectionFailedError(message)
+
+    def charge_authorization(self, transaction: Transaction):
+        data = transaction.to_json()
+        headers, _ = self.build_request_args()
+
+        response = requests.post(self.PAYSTACK_URL + self.__endpoint, headers=headers, data=data)
+        content = response.content
+        content = self.parse_response_content(content)
+
+
+        status, message = self.get_content_status(content)
+
+        #status = True for a successful connection
+        if status:
+            return content['data']
+        else:
+            #Connection failed
+            raise APIConnectionFailedError(message)
+
+    def get_total_transactions(self):
+        '''
+        Get total amount recieved from transactions
+        '''
+        headers, _ = self.build_request_args()
+        url = self.PAYSTACK_URL + self.__endpoint
+        url += '/totals'
+        response = requests.get(url, headers=headers)
+
+        content = response.content
+        content = self.parse_response_content(content)
+
+        status, message = self.get_content_status(content)
+
         if status:
             return content['data']
         else:
             raise APIConnectionFailedError(message)
 
-    @classmethod
-    def fromJSON(self, data):
+    def get_transactions(self):
         '''
-        Class method for converting JSON Object to PaymentManager Object
-
-        Arguments:
-        data : JSON serialized PaymentManager object
+        Gets all transactions
         '''
+        headers, _ = self.build_request_args()
 
-        manager_object = jsonpickle.decode(data)
+        response = requests.get(self.PAYSTACK_URL + self.__endpoint, headers=headers)
 
-        if type(manager_object) is PaymentManager:
-            return manager_object
+        content = response.content
+        content = self.parse_response_content(content)
 
+        status, message = self.get_content_status(content)
+
+        if status:
+            return content['data']
         else:
-            raise InvalidInstance('PaymentManager')
+            raise APIConnectionFailedError(message)
+
+    def get_transaction(self, id):
+        '''
+        Gets transaction with the specified id
+        '''
+        headers, data = self.build_request_args()
+
+        url = self.PAYSTACK_URL + self.__endpoint
+        url += '/%s' % (str(id))
+        response = requests.get(url, headers=headers)
+
+        content = response.content
+        content = self.parse_response_content(content)
+
+        status, message = self.get_content_status(content)
+
+        if status:
+            return content['data']
+        else:
+            raise APIConnectionFailedError(message)
 
 
-    def __str__(self):
-        return "Payment manager for %s" % (self.email)
-
+    def filter_transactions(self, amount_range: range, transactions):
+        '''
+        Returns all transactions with amounts in the given amount_range
+        '''
+        results = []
+        for transaction in transactions:
+            if Filter.filter_amount(amount_range, transaction):
+                results.append(transaction)
 
+        return results
 
 class CustomersManager(Manager):
     '''
@@ -327,39 +211,28 @@ class CustomersManager(Manager):
     __endpoint : Paystack API endpoint for 'customers' actions
 
     '''
-    __endpoint = None
+    __endpoint = '/customer'
 
-    def __init__(self, endpoint='/customer'):
+    def __init__(self):
         super().__init__(self)
-        self.__endpoint = endpoint
 
-    def create_customer(self, customer: Customer, meta={}):
+    def create_customer(self, customer: Customer, meta=None):
         '''
         Method for creating a new customer.
 
         Arguments :
         email  : Customer's email address
-        first_name (optional)
-        last_name (optional)
+        customer : Customer object
         meta : Dict which can contain additional customer information
         '''
 
-        if not type(customer) is Customer:
+        if not isinstance(customer, Customer):
             raise TypeError("customer argument should be an instance of the Customer class")
 
         if validators.email(customer.email):
-            data = {'email' : customer.email}
+            data = customer.to_json(pickled=False)
 
-            if first_name:
-                data['first_name'] = customer.first_name
-
-            if last_name:
-                data['last_name'] = customer.last_name
-
-            if meta:
-                data['metadata'] = meta
-
-            headers, data = self.build_request_args(data)
+            headers, _ = self.build_request_args()
 
         else:
             raise InvalidEmailError
@@ -387,7 +260,7 @@ class CustomersManager(Manager):
         '''
         Method which returns all registered customers
         '''
-        headers, data = self.build_request_args()
+        headers, _ = self.build_request_args()
 
         response = requests.get(self.PAYSTACK_URL + self.__endpoint, headers=headers)
 
@@ -397,22 +270,28 @@ class CustomersManager(Manager):
         status, message = self.get_content_status(content)
 
         if status:
-            return content['data']
+            data = content['data']
+            meta = content['meta']
+            customers = []
+            for item in data:
+                customer = Customer.from_json(item, pickled=False)
+                customers.append(customer)
+                return (customers, meta)
 
         else:
             raise APIConnectionFailedError(message)
 
 
-    def get_customer(self, id):
+    def get_customer(self, customer_id):
         '''
-        Method for getting a particular customer with the specified id
+        Method for getting a particular customer with the specified customer_id
 
         Arguments :
-        id : Customer id
+        customer_id : Customer id
 
         '''
-        headers, data = self.build_request_args()
-        url = "%s%s/%s" % (self.PAYSTACK_URL, self.__endpoint, id)
+        headers, _ = self.build_request_args()
+        url = "%s%s/%s" % (self.PAYSTACK_URL, self.__endpoint, customer_id)
         response = requests.get(url, headers=headers)
 
         content = response.content
@@ -421,12 +300,13 @@ class CustomersManager(Manager):
         status, message = self.get_content_status(content)
 
         if status:
-            customer = Customer.fromJSON(content['data'])
+            data = content['data']
+            customer = Customer.from_json(data, pickled='False')
             return customer
         else:
             raise APIConnectionFailedError(message)
 
-    def update_customer(self, id, data):
+    def update_customer(self, customer_id, updated_customer: Customer):
         '''
         Method for updating an existing customer
 
@@ -435,8 +315,12 @@ class CustomersManager(Manager):
         data : Dict which contains the values to be updated as keys
                 and the updated information as the values
         '''
-        headers, data = self.build_request_args(data)
-        url = "%s%s/%s" % (self.PAYSTACK_URL, self.__endpoint, id)
+        if not isinstance(updated_customer, Customer):
+            raise TypeError("customer argument should be of type 'Customer' ")
+
+        data = updated_customer.to_json(pickled=False)
+        headers, _ = self.build_request_args()
+        url = "%s%s/%s" % (self.PAYSTACK_URL, self.__endpoint, customer_id)
 
         response = requests.put(url, headers=headers, data=data)
         content = response.content
@@ -444,27 +328,31 @@ class CustomersManager(Manager):
 
         status, message = self.get_content_status(content)
         if status:
-            return content['data']
+            data = content['data']
+            return Customer.from_json(data)
         else:
             raise APIConnectionFailedError(message)
 
-    def set_risk_action(self, risk_action, id):
+    def set_risk_action(self, risk_action, customer: Customer):
         '''
         Method for either blacklisting or whitelisting a customer
 
         Arguments :
         risk_action : (allow or deny)
-        id : Customer id
+        customer_id : Customer id
 
         '''
 
+        if not isinstance(customer, Customer):
+            raise TypeError("customer argument should be of type 'Customer' ")
+
         endpoint = '/set_risk_action'
 
         if risk_action not in ('allow', 'deny'):
             raise ValueError("Invalid risk action")
 
         else:
-            data = {'customer' : id, 'risk_action' : risk_action}
+            data = {'customer' : customer.id, 'risk_action' : risk_action}
             headers, data = self.build_request_args(data)
             url = "%s%s" % (self.PAYSTACK_URL + self.__endpoint, endpoint)
 
@@ -476,7 +364,8 @@ class CustomersManager(Manager):
             status, message = self.get_content_status(content)
 
             if status:
-                return content['data']
+                data = content['data']
+                return Customer.from_json(data)
             else:
                 raise APIConnectionFailedError(message)
 
@@ -528,28 +417,28 @@ class PlanManager(Manager):
         '''
         url = self.PAYSTACK_URL + self.__endpoint
 
-        data = {
-            'name' : plan.name,
-            'interval' : plan.interval,
-            'amount' : plan.amount
-            }
+        data = plan.to_json()
 
-        headers, data = self.build_request_args(data)
+        headers, _ = self.build_request_args()
 
-        response = requests.post(url, headers = headers, data = data)
+        response = requests.post(url, headers=headers, data=data)
         content = response.content
         content = self.parse_response_content(content)
 
         status, message = self.get_content_status(content)
 
         if status:
-            return content['data']
-        else:            
+            data = content['data']
+            return Plan.from_json(data)
+        else:
             raise APIConnectionFailedError(message)
 
     def get_plans(self):
-        headers, data = self.build_request_args()
-        response = requests.get(self.PAYSTACK_URL + self.__endpoint, headers = headers)
+        '''
+        Method for getting plans
+        '''
+        headers, _ = self.build_request_args()
+        response = requests.get(self.PAYSTACK_URL + self.__endpoint, headers=headers)
 
         content = response.content
         content = self.parse_response_content(content)
@@ -557,17 +446,24 @@ class PlanManager(Manager):
         status, message = self.get_content_status(content)
 
         if status:
-            return content['data']
+            data = content['data']
+            meta = content['meta']
+            plans = []
+            for item in data:
+                plan = Plan.from_json(item)
+                plans.append(plan)
+                return (plans, meta)
         else:
             raise APIConnectionFailedError(message)
 
+    def get_plan(self, plan_id):
+        '''
+        Method for getting a plan with the specified id
+        '''
+        headers, _ = self.build_request_args()
 
-    #Implement as class method to return a Plan object
-    def get_plan(self, id):
-        headers, data = self.build_request_args()
-
-        url = "%s%s/%s" % (self.PAYSTACK_URL,self.__endpoint, id)
-        response = requests.get(url, headers = headers)
+        url = "%s%s/%s" % (self.PAYSTACK_URL, self.__endpoint, plan_id)
+        response = requests.get(url, headers=headers)
 
         content = response.content
         content = self.parse_response_content(content)
@@ -575,115 +471,144 @@ class PlanManager(Manager):
         status, message = self.get_content_status(content)
 
         if status:
-            return content['data']
+            data = content['data']
+            plan = Plan.from_json(data)
+            return plan
+        else:
+            raise APIConnectionFailedError(message)
+
+    def update_plan(self, plan_id, updated_plan: Plan):
+        '''
+        Method for updating existing plan
+        '''
+        if not isinstance(updated_plan, Plan):
+            raise TypeError("updated_plan argument should be of type 'Plan' ")
+
+        data = updated_plan.to_json()
+        headers, _ = self.build_request_args()
+        url = "%s%s/%s" % (self.PAYSTACK_URL, self.__endpoint, plan_id)
+
+        response = requests.put(url, headers=headers, data=data)
+        content = response.content
+        content = self.parse_response_content(content)
+
+        status, message = self.get_content_status(content)
+        if status:
+            data = content['data']
+            return Plan.from_json(data)
         else:
             raise APIConnectionFailedError(message)
 
 
 
-class TransactionsManager(Manager):
-    
-    __endpoint = '/transaction'
-    def __init__(self, endpoint = '/transaction'):
+class TransfersManager(Manager):
+    '''
+    TransfersManager class
+    '''
+
+    __endpoint = '/transfer'
+    def __init__(self, endpoint='/transfer'):
         super().__init__(self)
         self.__endpoint = endpoint
 
-    def charge_authorization(self, authorization_code, amount, email, plan_code = None):
-        data = {'authorization_code' : authorization_code}
-        if plan_code:
-            data['plan'] = plan_code
 
-        try:
-            amount = int(amount)
-        except ValueError as error:
-            raise ValueError("Invalid amount. Amount(in kobo) should be an integer")
-        else:
-            if validators.email(email):
-                data['amount'] = amount
-                data['email'] = email
-                headers, data = self.build_request_args(data)
-            else:
-                raise InvalidEmailError
+    def initiate_transfer(self, transfer: Transfer):
+        '''
+        Method to start a transfer to a bank account.
+        '''
 
-        response = requests.post(self.PAYSTACK_URL + self.__endpoint, headers = headers, data = data)         
-        content = response.content              
-        content = self.parse_response_content(content)    
+        data = transfer.to_json()
+
+
+        headers, _ = self.build_request_args()
+
+        url = self.PAYSTACK_URL + self.__endpoint
+        response = requests.post(url, headers=headers, data=data)
+        content = response.content
+        content = self.parse_response_content(content)
 
 
         status, message = self.get_content_status(content)
 
-        #status = True for a successful connection
         if status:
-            return content['data']
+            data = content['data']
+            transfer = Transfer.from_json(data)
+            return transfer
+
         else:
             #Connection failed
-            raise APIConnectionFailedError(message)       
-        
-    def get_total_transactions(self):
-        '''
-        Get total amount recieved from transactions
-        '''
-        headers, data = self.build_request_args()
-        url = self.PAYSTACK_URL + self.__endpoint
-        url += '/totals'
-        response = requests.get(url, headers = headers)
+            raise APIConnectionFailedError(message)
 
+
+    def get_transfers(self):
+        '''
+        Method to get all paystack transfers
+        '''
+        headers, _ = self.build_request_args()
+
+        url = self.PAYSTACK_URL + self.__endpoint
+        response = requests.get(url, headers=headers)
         content = response.content
         content = self.parse_response_content(content)
 
         status, message = self.get_content_status(content)
 
         if status:
-            return content['data']
+            data = content['data']
+            transfers = []
+            for item in data:
+                transfers.append(Transfer.from_json(item))
+
+            return transfers
+
         else:
+            #Connection failed
             raise APIConnectionFailedError(message)
 
-    def get_transactions(self):
+    def get_transfer(self, transfer_id):
         '''
-        Gets all transactions
+        Method to get paystack transfer with the specified id
         '''
         headers, data = self.build_request_args()
 
-        response = requests.get(self.PAYSTACK_URL + self.__endpoint, headers = headers)
-
+        url = self.PAYSTACK_URL + self.__endpoint
+        url += '/%s' % (transfer_id)
+        response = requests.post(url, headers=headers)
         content = response.content
         content = self.parse_response_content(content)
 
         status, message = self.get_content_status(content)
 
         if status:
-            return content['data']
+            data = content['data']
+            return Transfer.from_json(data)
         else:
+            #Connection failed
             raise APIConnectionFailedError(message)
 
-    def get_transaction(self, id):
+    def finalize_transfer(self, transfer_id, otp):
         '''
-        Gets transaction with the specified id
+        Method for finalizing transfers
         '''
-        headers, data = self.build_request_args()        
+        transfer_id = str(transfer_id)
+        otp = str(otp)
+
+        data = {'transfer_code' : transfer_id, 'otp' : otp}
+        headers, data = self.build_request_args(data)
 
         url = self.PAYSTACK_URL + self.__endpoint
-        url += '/%s' % (str(id))
-        response = requests.get(url, headers = headers)
-
+        url += '/finalize_transfer'
+        response = requests.post(url, headers=headers, data=data)
         content = response.content
         content = self.parse_response_content(content)
 
+
         status, message = self.get_content_status(content)
 
         if status:
-            return content['data']
+            data = content['data']
+            return data
+
         else:
+            #Connection failed
             raise APIConnectionFailedError(message)
-
-    
-    def filter_transactions(self, amount_range : range, transactions):
-        '''
-        Returns all transactions with amounts in the given amount_range
-        '''
-        results = []
-        for transaction in transactions:
-            if Filter.filter_amount(amount_range, transaction):
-                results.append(transaction)
-
-        return results
\ No newline at end of file
